#!/bin/bash

SCRIPT="$(readlink -f "$0")"
SCRIPT_DIR="$(dirname "$SCRIPT")"

COMMON_SCRIP_DIR="$SCRIPT_DIR/common"
LIB_SCRIPT_DIR="$COMMON_SCRIP_DIR/lib"

source "$LIB_SCRIPT_DIR/globals.sh"
source "$LIB_SCRIPT_DIR/utils.sh"

function restore() {
    usermod -G $(cat "$BK_DIR/wwwdata_groups") www-data
    usermod -G $(cat "$BK_DIR/postgres_groups") postgres
    setfacl --restore="$BK_DIR/macdb_perm.facl"
    rm -r "$BK_DIR"
}

function remove_ald() {
    expect << EOF
spawn ald-init destroy
expect -exact "Вы УВЕРЕНЫ, что хотите ВЫПОЛНИТЬ эту операцию? (yes/no) \[no\]: "
send -- "yes\r"
expect eof
EOF

    file="/etc/hostname"
    mv "${file}.${BK_POSTFIX}" $file

    file="/etc/hosts"
    mv "${file}.${BK_POSTFIX}" $file

    file="/etc/network/interfaces"
    mv "${file}.${BK_POSTFIX}" $file

    ip link del dummy0 type dummy0
    ip link delete $ETH_INTERFACE

    rm -f /etc/modules-load.d/dummy.conf
    rm -f /etc/modprobe.d/dummy.conf

    /etc/init.d/networking restart
}

function reset_sssd() {
    local sssd_conf="/etc/sssd/sssd.conf"
    [[ ! -f ${sssd_conf} ]] && return

    info_d "###### Сброс sssd ######"

    sudo service sssd stop
    cp "${sssd_conf}.setestbk" "${sssd_conf}"
    sudo service sssd start
}

function remove_pg() {
    local pg_ver
    pg_ver=$(pg_version)

    /etc/init.d/postgresql stop
    sleep 3
    pg_dropcluster $pg_ver contrprimer
    rm -f "$PG_KEYTAB"
    /etc/init.d/postgresql start
}

function remove_test_sites() {
    local path_conf="/etc/apache2/conf-available"
    local path_sites="/etc/apache2/sites-available"

    /etc/init.d/apache2 stop

    a2dissite se-tests-pam.conf
    a2dissite se-tests-krb.conf
    
    cp ${path_conf}/security.conf.setestbak ${path_conf}/security.conf
    rm -f ${path_sites}/se-tests-pam.conf
    rm -f ${path_sites}/se-tests-krb.conf
    rm -f "$APACHE_KEYTAB"
    
    sed -i 's/AstraMode off//g' /etc/apache2/apache2.conf
    # TODO: очистить последствия 
    # pdpl-user -l 0:3 www-data
    /etc/init.d/apache2 start
}

function remove_test_files() {
    rm -rf "$TEST_PATH"
    rm -f "${USR_TEST_DIR}/diffs"/*
    rm -f "${USR_TEST_DIR}/results"/*
}

function remove_local_users() {
    deluser --remove-home apachelocaluser1
    deluser --remove-home apachelocaluser2
}

function main() {
    validate_user

    restore

    info_d "###### Удаление ald-домена ######"
    remove_ald

    reset_sssd

    info_d "###### Удаление postgresql кластера ######"
    remove_pg

    info_d "###### Удаление тестовых сайтов ######"
    remove_test_sites

    info_d "###### Удаление тестовых файлов ######"
    remove_test_files

    remove_local_users
}

main
