#!/bin/bash

RED='\033[0;31m'
GREEN='\033[0;32m'
NORMAL='\033[0m'
tput sgr0 

TEST_NUM=0
TEST_SUCCESS_COUNT=0
TEST_ERROR_COUNT=0

SERVER_NAME="srv1.test.ru"
RESULT_PATH="/usr/share/apache2/test"
CMD_CURL_NEG="/tmp/curl_negot_req.sh"
CMD_CURL="/tmp/curl_req.sh"

function stop_apache2() {
    echo "Stopping apache2..."
    /etc/init.d/apache2 stop
    sleep 2
    while pidof apache2 > /dev/null; do
        echo "Kill apache2"
        kill -INT $(pidof apache2)
        sleep 1
    done
}

function start_apache2() {
    echo "Starting apache2..."
    /etc/init.d/apache2 start
    local i=0
    while [ $i -lt 10 ]; do
        pidof apache2 > /dev/null && break
        sleep 1
        i=$((i+1))
    done
    if [ $i == 10 ]; then
        echo "Failed to start apache2!" 
    fi    
}

function do_test() {
    local -n _mlvls=$1
    local -n _mcats=$2
    local -n _requests=$3
    local -n _users=$4
    local cmd=$5
    local postfix=$6

    for mlvl in "${_mlvls[@]}"; do
        for mcat in "${_mcats[@]}"; do
            for request in "${_requests[@]}"; do
                for user in "${_users[@]}"; do
                    TEST_NUM=$((TEST_NUM+1))
                    echo -n "Тест номер $TEST_NUM. Запрос $request от пользователя $user (m:$mlvl; c:$mcat) "
                    rslt_file_name="${request//\//_}"-$user-$mlvl-$mcat$postfix
                    apache2-se-tests --command "$cmd $request $user" -m "$mlvl" -c "$mcat" -u "$user" > "$RESULT_PATH/results/$rslt_file_name"

                    if grep -q "<title>" "$RESULT_PATH/results/$rslt_file_name"; then
                        rslt_status=$(cat "$RESULT_PATH/results/$rslt_file_name" | sed -n 's/.*<title> \(.*\) <\/title>.*/\1/p')
                        exp_status=$(cat "$RESULT_PATH/expected/$rslt_file_name" | sed -n 's/.*<title> \(.*\) <\/title>.*/\1/p')  
                        if [ "$rslt_status" != "$exp_status" ]; then
                            diff -C3 -N "$RESULT_PATH/expected/$rslt_file_name" "$RESULT_PATH/results/$rslt_file_name" > "$RESULT_PATH/diffs/$rslt_file_name"
                            echo -e "[$RED Error $NORMAL]"
                            TEST_ERROR_COUNT=$((TEST_ERROR_COUNT+1))
                        else
                            echo -e "[$GREEN Done $NORMAL]"
                            TEST_SUCCESS_COUNT=$((TEST_SUCCESS_COUNT+1))
                        fi
                    else
                        diff_res=$(diff -q -N "$RESULT_PATH/expected/$rslt_file_name" "$RESULT_PATH/results/$rslt_file_name")
                        if [ "x$diff_res" != "x" ]; then
                            diff -C3 -N "$RESULT_PATH/expected/$rslt_file_name" "$RESULT_PATH/results/$rslt_file_name" > "$RESULT_PATH/diffs/$rslt_file_name"
                            echo -e "[$RED Error $NORMAL]"
                            TEST_ERROR_COUNT=$((TEST_ERROR_COUNT+1))
                        else
                            echo -e "[$GREEN Done $NORMAL]"
                            TEST_SUCCESS_COUNT=$((TEST_SUCCESS_COUNT+1))
                        fi
                    fi
                done
            done
        done
    done
}


function test_krb5_auth_amo() {
    local mlvls=( 0 1 2 3 )
    local mcats=( 0 0x1 0x2 0x4 0x8 )

    local requests=( "http://$SERVER_NAME/d0/f0"
"http://$SERVER_NAME/d1/f0"
"http://$SERVER_NAME/d1/f1"
"http://$SERVER_NAME/d2/f0"
"http://$SERVER_NAME/d2/f1"
"http://$SERVER_NAME/d2/f2"
"http://$SERVER_NAME/d3/f0"
"http://$SERVER_NAME/d3/f1"
"http://$SERVER_NAME/d3/f2"
"http://$SERVER_NAME/d3/f3"
"http://$SERVER_NAME/index.php" )

    local users=(user_ruk user_mo user_fso user_fsb)

    echo
    echo "### Тесты с kerberos аутентификацией ### AstraMode on "
    echo

    do_test mlvls mcats requests users "$CMD_CURL_NEG" ""
}

function test_pam_auth_amo() {
    local mlvls=( 0 1 2 3 )
    local mcats=( 0 0x1 0x2 0x4 0x8 )

    local requests=("http://$SERVER_NAME:81/d0/f0"
"http://$SERVER_NAME:81/d1/f0"
"http://$SERVER_NAME:81/d1/f1"
"http://$SERVER_NAME:81/d2/f0"
"http://$SERVER_NAME:81/d2/f1"
"http://$SERVER_NAME:81/d2/f2"
"http://$SERVER_NAME:81/d3/f0"
"http://$SERVER_NAME:81/d3/f1"
"http://$SERVER_NAME:81/d3/f2"
"http://$SERVER_NAME:81/d3/f3"
"http://$SERVER_NAME:81/index.php"
)

    local users=( apachelocaluser1 apachelocaluser2 )

    echo
    echo "### Тесты с pam аутентификацией ### AstraMode on "
    echo

    do_test mlvls mcats requests users "$CMD_CURL" ""
}

function test_krb5_auth_amf() {
    local mlvls=( 0 )
    local mcats=( 0 )

    local requests=("http://$SERVER_NAME/d0/f0"
"http://$SERVER_NAME/d1/f0"
"http://$SERVER_NAME/d1/f1"
"http://$SERVER_NAME/d2/f0"
"http://$SERVER_NAME/d2/f1"
"http://$SERVER_NAME/d2/f2"
"http://$SERVER_NAME/d3/f0"
"http://$SERVER_NAME/d3/f1"
"http://$SERVER_NAME/d3/f2"
"http://$SERVER_NAME/d3/f3"
"http://$SERVER_NAME/index.php"
)

    local users=( user_ruk user_mo user_fso user_fsb user_db )

    echo
    echo "### Тесты с kerberos аутентификацией ### AstraMode off "
    echo

    do_test mlvls mcats requests users "$CMD_CURL_NEG" "-AMF"
}

function test_pam_auth_amf() {
    local mlvls=( 0 )
    local mcats=( 0 )

    local requests=( "http://$SERVER_NAME:81/d0/f0"
"http://$SERVER_NAME:81/d1/f0"
"http://$SERVER_NAME:81/d1/f1"
"http://$SERVER_NAME:81/d2/f0"
"http://$SERVER_NAME:81/d2/f1"
"http://$SERVER_NAME:81/d2/f2"
"http://$SERVER_NAME:81/d3/f0"
"http://$SERVER_NAME:81/d3/f1"
"http://$SERVER_NAME:81/d3/f2"
"http://$SERVER_NAME:81/d3/f3"
"http://$SERVER_NAME:81/index.php" )


    local users=( apachelocaluser1 apachelocaluser2 )

    echo
    echo "### Тесты с pam аутентификацией ### AstraMode off "
    echo

    do_test mlvls mcats requests users "$CMD_CURL" "-AMF"
}

function test_overrides_astramode() {
    local path_sites="/etc/apache2/sites-available"

    echo
    echo "Тестирование переопределения AstraMode (global on, virtual hosts off) "
    echo

    stop_apache2
    sed -i 's/<\/VirtualHost>/\tAstraMode off\n<\/VirtualHost>/g' "${path_sites}/se-tests-pam.conf"
    sed -i 's/<\/VirtualHost>/\tAstraMode off\n<\/VirtualHost>/g' "${path_sites}/se-tests-krb.conf"
    start_apache2

    test_krb5_auth_amf
    test_pam_auth_amf 
    
    echo
    echo "Тестирование переопределения AstraMode (global on, virtual hosts on/off) "
    echo

    stop_apache2
    sed -i 's/AstraMode off/AstraMode on/g' "${path_sites}/se-tests-pam.conf"
    start_apache2

    test_krb5_auth_amf
    test_pam_auth_amo 

    
    echo
    echo "Тестирование переопределения AstraMode (global off, virtual hosts on) "
    echo

    stop_apache2
    echo "AstraMode off" >> /etc/apache2/apache2.conf
    sed -i 's/AstraMode off/AstraMode on/g' "${path_sites}/se-tests-krb.conf"
    start_apache2

    test_krb5_auth_amo
    test_pam_auth_amo

    echo
    echo "Тестирование переопределения AstraMode (global off, virtual hosts on/off) "
    echo

    stop_apache2
    sed -i 's/AstraMode on/AstraMode off/g' "${path_sites}/se-tests-pam.conf"
    start_apache2

    test_krb5_auth_amo
    test_pam_auth_amf 

    # clean
    stop_apache2
    sed -i '/AstraMode off/d' /etc/apache2/apache2.conf
    sed -i '/AstraMode off/d' "${path_sites}/se-tests-pam.conf"
    sed -i '/AstraMode on/d' "${path_sites}/se-tests-krb.conf"
    start_apache2
}

function prepare_path() {
    local path="$1"
    if [[ -d $path ]]; then
        rm -f "$path"/*
    else
        mkdir -p "$path"
    fi    
}


function main() {
    RESULT_PATH=$1
    SERVER_NAME=$2
    CMD_CURL_NEG=$3
    CMD_CURL=$4

    # prepare
    prepare_path "$RESULT_PATH/results"
    prepare_path "$RESULT_PATH/diffs"
    start_apache2
 
    # AstraMode on
    test_krb5_auth_amo
    test_pam_auth_amo

    echo
    echo "Тестирование AstraMode off "
    echo

    stop_apache2
    echo "AstraMode off" >> /etc/apache2/apache2.conf
    start_apache2

    # AstraMode off
    test_krb5_auth_amf
    test_pam_auth_amf 

    # clean
    stop_apache2
    sed -i '/AstraMode off/d' /etc/apache2/apache2.conf
    start_apache2

    # AstraMode override tests (since 2.4.25-astrase19.0.11)
    apache2_ver=$(dpkg-query --show --showformat '${Version}' apache2)
    if $(dpkg --compare-versions $apache2_ver ge "2.4.25-astrase19.0.11"); then
        test_overrides_astramode
    fi

    # results
    echo -en "Всего тестов $TEST_NUM из них ошибочных"

    if [ $TEST_ERROR_COUNT -eq 0 ]; then
        echo -e " $GREEN${TEST_ERROR_COUNT} $NORMAL"
        tput sgr0 
        exit 0
    else
        echo -e " $RED${TEST_ERROR_COUNT} $NORMAL"
        tput sgr0 
        exit 1
    fi       
}

main "$1" "$2" "$3" "$4"
