#!/bin/bash

SCRIPT="$(readlink -f "$0")"
SCRIPT_DIR="$(dirname "$SCRIPT")"

COMMON_SCRIP_DIR="$SCRIPT_DIR/common"
LIB_SCRIPT_DIR="$COMMON_SCRIP_DIR/lib"

source "$LIB_SCRIPT_DIR/globals.sh"
source "$LIB_SCRIPT_DIR/utils.sh"

PWD_FILE_NAME="${USR_TEST_DIR}/passwords.txt"
FORCE_OPT="-f --pass-file=$PWD_FILE_NAME"
PG_PORT=9060
PG_VERSION=$(pg_version)

# Функция создает и наполняет файл текстом в соответствии с параметрами
# $1 - путь до файла
# $2 - мандатный уровень
function mkhtmlfile {
	cat <<EOT >> "$1/f$2"
<!DOCTYPE html>
<html>
	<head>
	    <meta http-equiv="content-type" content="text/html; charset=utf-8"/>
	    <title>Выберите файлы</title>
	</head>
	<body>
		<h1>It works!</h1>
		<p>File mac label is $2</p>
	</body>
</html>
EOT

	pdpl-file "$2"::: "$1/f$2"
}

function backup() {
    mkdir -p "$BK_DIR"
    id -nG www-data | sed 's/\ /,/g' > "$BK_DIR/wwwdata_groups"
    id -nG postgres | sed 's/\ /,/g' > "$BK_DIR/postgres_groups"
    getfacl -p /etc/parsec/macdb > "$BK_DIR/macdb_perm.facl"
}

function gen_test_files() {
	local idx=$1
	local path="$TEST_PATH/d${idx}"

	mkdir "$path"
	pdpl-file "${idx}":::ccnr,ccnri "$path"
	
	for ((i=0; i<=idx; i++)); do
		mkhtmlfile "$path" $i	
	done
}

function ald_create_user() {
	local user=$1
	local min_cat_hex=$2
	local max_cat_hex=$3

	ald-admin user-add $user $FORCE_OPT
	ald-admin user-ald-cap $user --add-hosts --host="$SERVER_NAME" $FORCE_OPT
	ald-admin user-mac $user --min-lev-int=0 --max-lev-int=3 --min-cat-hex=$min_cat_hex --max-cat-hex=$max_cat_hex $FORCE_OPT	
}

function configure_ald() {
	file=/etc/hostname
	cp $file "$file.$BK_POSTFIX"
	echo "$SERVER_NAME" > "/etc/hostname"

	hostname srv1

	file="/etc/hosts"
	cp $file "$file.$BK_POSTFIX"
	cat <<EOT > $file
127.0.0.1	localhost
192.168.0.1	$SERVER_NAME	srv1
EOT

	modprobe dummy
	ip link add $ETH_INTERFACE type dummy

	file="/etc/network/interfaces"
	cp $file "$file.$BK_POSTFIX"
	cat <<EOT >> $file

auto $ETH_INTERFACE
iface $ETH_INTERFACE inet static
	address 192.168.0.1
	netmask 255.255.255.0

EOT

	echo "dummy" > /etc/modules-load.d/dummy.conf
	echo "install dummy /sbin/modprobe --ignore-install dummy; /sbin/ip link add $ETH_INTERFACE type dummy" > /etc/modprobe.d/dummy.conf

	/etc/init.d/networking restart

	cat << EOF  > "$PWD_FILE_NAME"
admin/admin:1
user_ruk:1
user_mo:1
user_fso:1
user_fsb:1
user_db:1
EOF

	chown root.root "$PWD_FILE_NAME"
	chmod 0600 "$PWD_FILE_NAME"

	expect "$COMMON_SCRIP_DIR"/expect_ald_init.exp

	ald-admin policy-mod default --min-length=1 --min-classes=1 $FORCE_OPT

	ald-admin cmd --pass-file="$PWD_FILE_NAME" << EOF
maclev-mod 0 Несекретно
maclev-mod 1 ДСП
maclev-mod 2 Секретно
maclev-mod 3 Сов.секретно

maccat-add 0x1 Минобороны
maccat-add 0x2 ФСО
maccat-add 0x4 ФСБ
maccat-add 0x8 КП
exit
EOF

}

function create_users() {
	"${COMMON_SCRIP_DIR}/expect_create_local_users.exp"

	pdpl-user -l 0:2 apachelocaluser1
	pdpl-user -l 0:3 apachelocaluser2

	ald_create_user user_ruk 0xf 0xf
	ald_create_user user_mo 0x1 0x1
	ald_create_user user_fso 0x2 0x2
	ald_create_user user_fsb 0x4 0x4
	ald_create_user user_db 0x0 0x4
}

function configure_sssd() {
	local sssd_conf="/etc/sssd/sssd.conf"
	[[ ! -f ${sssd_conf} ]] && return

	info_d "###### Настройка sssd ######"
	
	cp "${sssd_conf}" "${sssd_conf}.setestbk"

	sudo service sssd stop
	sed -i "s/allowed_uids = .*/allowed_uids = 0, user_ruk, user_mo, user_fso, user_fsb/" ${sssd_conf}
	sudo service sssd start	
}

function configure_pg() {
	ald-admin service-add postgres/"$SERVER_NAME" $FORCE_OPT
	ald-admin sgroup-svc-add postgres/"$SERVER_NAME" --sgroup=mac $FORCE_OPT
	ald-client update-svc-keytab postgres/"$SERVER_NAME" --ktfile="$PG_KEYTAB" $FORCE_OPT
	chown postgres "$PG_KEYTAB"
	chmod 644 "$PG_KEYTAB"

	usermod -a -G shadow postgres
	setfacl -d -m u:postgres:r /etc/parsec/macdb
	setfacl -R -m u:postgres:r /etc/parsec/macdb
	setfacl -m u:postgres:rx /etc/parsec/macdb

	pg_createcluster -p $PG_PORT $PG_VERSION contrprimer

	cat <<EOT > "/etc/postgresql/${PG_VERSION}/contrprimer/pg_hba.conf"
local	all		postgres		trust
host	all		all	127.0.0.1/32	trust
host	all		all	$SERVER_NAME	gss	include_realm=0
EOT

	file="/etc/postgresql/${PG_VERSION}/contrprimer/postgresql.conf"
	sed -i -e 's/^ac_ignore_socket_maclabel[[:space:]]*=.*/ac_ignore_socket_maclabel = false/' "$file"
	sed -i -e "s/^listen_addresses[[:space:]]*=.*/listen_addresses = '*'/" "$file"
	echo "krb_server_keyfile='$PG_KEYTAB'" >> "$file"

	/etc/init.d/postgresql restart

	psql -p $PG_PORT -h localhost -U postgres -c 'CREATE DATABASE contrprimer;'
	psql -p $PG_PORT -h localhost -U postgres -d contrprimer << EOF
MAC CCR ON CLUSTER IS OFF;
MAC CCR ON DATABASE contrprimer IS OFF;
MAC LABEL ON CLUSTER IS '{3,f}';
MAC LABEL ON DATABASE contrprimer is '{3,f}';
CREATE USER user_ruk;
grant ALL ON DATABASE contrprimer TO user_ruk;
CREATE USER user_mo;
grant ALL ON DATABASE contrprimer TO user_mo;
CREATE USER user_fso;
grant ALL ON DATABASE contrprimer TO user_fso;
CREATE USER user_fsb;
grant ALL ON DATABASE contrprimer TO user_fsb;
CREATE USER apachelocaluser1;
grant ALL ON DATABASE contrprimer TO apachelocaluser1;
CREATE USER apachelocaluser2;
grant ALL ON DATABASE contrprimer TO apachelocaluser2;
CREATE USER user_db;
grant ALL ON DATABASE contrprimer TO user_db;
\q
EOF

}

function initialize_pg_db() {
	psql -h localhost -U postgres -p $PG_PORT -d contrprimer < "${COMMON_SCRIP_DIR}/contrprimer.backup"
}

function configure_apache2() {
	ald-admin service-add HTTP/"$SERVER_NAME" $FORCE_OPT
	ald-admin sgroup-svc-add HTTP/"$SERVER_NAME" --sgroup=mac $FORCE_OPT
	ald-client update-svc-keytab HTTP/"$SERVER_NAME" --ktfile="$APACHE_KEYTAB" $FORCE_OPT
	chown www-data "$APACHE_KEYTAB"
	chmod 644 "$APACHE_KEYTAB"

	usermod -a -G shadow www-data
	setfacl -d -m u:www-data:r /etc/parsec/macdb
	setfacl -R -m u:www-data:r /etc/parsec/macdb
	setfacl -m u:www-data:rx /etc/parsec/macdb

	pdpl-user -l 0:3 www-data
}

function create_sites_conf() {
	local path_conf="/etc/apache2/conf-available"
    local path_sites="/etc/apache2/sites-available"

	/etc/init.d/apache2 stop
	
	a2dissite 000-default.conf

	cp "${COMMON_SCRIP_DIR}/se-tests-pam.conf" "${path_sites}/se-tests-pam.conf"
	cp "${COMMON_SCRIP_DIR}/se-tests-krb.conf" "${path_sites}/se-tests-krb.conf"

	a2ensite se-tests-pam.conf
	a2ensite se-tests-krb.conf
	
	cp "${path_conf}/security.conf" "${path_conf}/security.conf.setestbak"
	cp "${COMMON_SCRIP_DIR}/security.conf" "${path_conf}/security.conf"
	
	/etc/init.d/apache2 start
}

function create_test_files() {
	rm -rf "$TEST_PATH"
	mkdir "$TEST_PATH"
	pdpl-file 3:::ccnr,ccnri "$TEST_PATH"

	for i in {0..3}; do
		gen_test_files $i
	done

	cp "${COMMON_SCRIP_DIR}/index.php" "$TEST_PATH"

	chown -R www-data:www-data "$TEST_PATH"
	chmod 744 "${TEST_PATH}/index.php"
}

function main() {
	validate_user

	backup

	info_d "###### Конфигурирование ald-домена ######"
	configure_ald

        info_d "###### Создание пользователей ######"
	create_users

	configure_sssd

	info_d "###### Настройка сервиса postgresql ######"
	configure_pg

	info_d "###### Инициализация базы данных postgres ######"
	initialize_pg_db

	info_d "###### Настройка сервиса apache2 ######"
	configure_apache2

	info_d "###### Создание конфигураций для сайтов ######"
	create_sites_conf

	info_d "###### Создание тестовых файлов ######"
	create_test_files

	rm "$PWD_FILE_NAME"
}

main
